require_relative '../db/models/task'

module Trackington
  class TaskRepository
    STATUSES = %w(open in_progress resolved closed reopened)
    PRIORITIES = %w(blocker critical major minor trivial)
    TYPES = %w(bug improvement new_feature story task)

    def initialize(sprint_id)
      @sprint_id = sprint_id
    end

    def move_to(sprint_id)
      tasks = Models::Task.where(sprint_id: sprint_id)

      tasks.each do |task|
        task.sprint_id = sprint_id if active?(task.status)
        task.save
      end
    end

    def add(data)
      data[:sprint_id] = @sprint_id
      task = Models::Task.new(data)
      task.save
    end

    def all
      Models::Task.where(sprint_id: @sprint_id).map do |db_task|
        Task.new(db_task)
      end
    end

    def get(id)
      db_task = Models::Task.where(sprint_id: @sprint_id, id: id).first
      Task.new(db_task)
    end

    def status_types
      STATUSES
    end

    def priorities
      PRIORITIES
    end

    def types
      TYPES
    end

    private

    def active?(status)
      %w(in_progress reopened open).include? status
    end
  end

  class Task
    STATUSES = %w(open in_progress resolved closed reopened)
    PRIORITIES = %w(blocker critical major minor trivial)
    TYPES = %w(bug improvement new_feature story task)

    attr_reader :id, :title, :description, :time_planned, :time_actual
    attr_reader :priority, :task_type, :status, :assigned_to, :created_by

    def initialize(db_task)
      basic_properties(db_task)
      status_properties(db_task)
      person_properties(db_task)
    end

    def active?
      %w(in_progress reopened open).include? @status
    end

    def update(data)
      db_task = Models::Task.find(@id)

      data.each do |property, new_value|
        db_task[property] = get_value(property, new_value) if property != :id
      end

      db_task.save

      basic_properties(db_task)
      status_properties(db_task)
      person_properties(db_task)
    end

    private

    def basic_properties(db_task)
      @id = db_task.id
      @title = db_task.title
      @description = db_task.description
    end

    def status_properties(db_task)
      @time_planned = db_task.time_planned
      @time_actual =  db_task.time_actual

      @priority = db_task.priority
      @task_type = db_task.task_type
      @status = db_task.status
    end

    def person_properties(db_task)
      @assigned_to = db_task.assigned_to
      @created_by = db_task.created_by
    end

    def get_value(property, value)
      case property
      when :status then STATUSES.index(value)
      when :priority then PRIORITIES.index(value)
      when :task_type then TYPES.index(value)
      else value
      end
    end
  end
end
